//=============================================================================
// ANNADeathMatch. (C) 2012 "Dei" & Nobody. Edit by Cozmo, 2018
//
// Any non-team gamemode should use this gametype. DM, ADM, custom maps etc
//=============================================================================
class CDX_DeathMatch extends MTLDeathMatch;

var string GameTypeName;
var string recentIP[40];
var int suspicion[40];
var float lastConnectTime[40];
var bool bHasGameFinished;
var ANNAManager Manager;
var ANNAGameMode AGM;
var CDXServerQuery CSQ;

/** A new structure which extends teh new Scoreboard with informations */
struct PlayerInfo
{
     var int ping;
};

var PlayerInfo PInfo[32]; //Array of the additional structure for 32 players

/** @ignore */
const IDX = 0.14;
/** @ignore */
const PINGX = 0.85;

function PostBeginPlay()
{
	Super.PostBeginPlay();
		
	Spawn(class'MutPlayerID');
	
	if(GameReplicationInfo == none)
		GameReplicationInfo = Spawn(GameReplicationInfoClass);

	HandleCSQ();
	HandleAGM();
}

function HandleCSQ()
{
	local UdpServerQuery UdpQuery;
	
	if(Level.NetMode != NM_Standalone)
	{
		foreach AllActors(class'UdpServerQuery', UdpQuery)
		{
			if(UdpQuery != none)
				break;
		}    

		if(UdpQuery == none)
		{
			CSQ = Spawn(class'CDXServerQuery');
		}
		else
		{
			if(CDXServerQuery(UdpQuery) == none)
			{
				UdpQuery.Destroy();
				CSQ = Spawn(class'CDXServerQuery');
			}
		}
	}
}

function HandleAGM()
{
	local DeusExMPGame gameClass;
	local ANNAGameMode agm2;
	local Pawn P;
	
	foreach AllActors(class'ANNAGameMode', agm2) 
		AGM = agm2;
	
	//create an AGM if the map doesn't have one  
	if(AGM == None)
		AGM = Spawn(class'ANNAGameMode'); //todo: make this spawn the correct AGM according to maplist or something
	
	//set player AGM
	for(P=Level.PawnList; P!=None; P=P.NextPawn)
	{
		if(P.IsA('ANNAPlayer'))
		{
			ANNAPlayer(P).AGM = AGM;
		}
	}

	//serverquery
	CSQ.AGM = AGM;
	
	//maplist?

	//scoreboard? idk, maybe it's better for AGM to interact with scoreboard indirectly, via GI and and player vars
}

function Trigger(actor Other, pawn EventInstigator )
{
	if(Event == 'newagm')
		HandleAGM();
	
	Event = '';
}

/**
  Sets the MPL depending on the just loaded map.
*/
event InitGame( string Options, out string Error )
{
    local ANNAManager curManager;
    local bool bFound;
    
    Super.InitGame(Options,Error);

    foreach AllActors(class'ANNAManager', curManager)
      if(curManager != None)
          bFound = true;

    if(!bFound)
      Manager = Spawn(class'ANNAManager');
    else
      Manager = curManager;

    CBPMutator(level.Game.BaseMutator).AddCBPMutator(Spawn(class'ANNAMutator'));
}

/**
  Find and return the ANNAManager
*/
function ANNAManager findManager()
{
  local ANNAManager curManager;
  local bool bFound;

  if(Manager != None)
    return Manager;

  foreach AllActors(class'ANNAManager', curManager)
    if(curManager != None)
      return curManager;

  return none;
}

simulated function ShowVictoryConditions( GC gc, float screenWidth, float yoffset, DeusExPlayer thisPlayer )
{
	local String str, secStr;
	local float x, y, w, h;
	local int timeLeft, minutesLeft, secondsLeft;
	local float ftimeLeft;

	if ( VictoryCondition ~= "Frags" )
		str = VictoryConString1 $ ScoreToWin $ VictoryConString2;
	else if ( VictoryCondition ~= "Time" )
	{
		timeLeft = ScoreToWin * 60 - Level.Timeseconds - thisPlayer.ServerTimeDiff;
		if ( timeLeft < 0 )
			timeleft = 0;
		minutesLeft = timeLeft/60;
		ftimeLeft = float(timeLeft);
		secondsLeft = int(ftimeLeft%60);
		if ( secondsLeft < 10 )
			secStr = "0" $ secondsLeft;
		else
			secStr = "" $ secondsLeft;

		str = TimeLimitString1 $ minutesLeft $ ":" $ secStr $ TimeLimitString2;
	}
//	else
//		log( "Warning: Unknown victory type:"$VictoryCondition$" " );

	gc.GetTextExtent( 0, w, h, str );
	x = (screenWidth * 0.5) - (w * 0.5);
	gc.DrawText( x, yoffset, w, h, str );
}

function bool CheckIPSpam(string Address)
{
	local int i, j;
	
	// strip port number
	j = InStr(Address, ":");
	
	if(j != -1)
		Address = Left(Address, j);
	
	for(i = 0; i < 40; i ++)
	{
		if(recentIP[i] == Address)
		{
			if(Level.TimeSeconds <= lastConnectTime[i] + 20)
			{
				suspicion[i] ++;
			}
			
			lastConnectTime[i] = Level.TimeSeconds;
			
			if(suspicion[i] >= 4)
			{
				return true;
				Log(Address$" was denied for reconnect spam");
			}
			else
			{
				return false;
			}
		}
	}
	
	//IP wasn't found. Add it
	for(i = 0; i < 40; i ++)
	{
		if(recentIP[i] == "")
		{
			recentIP[i] = Address;
			lastConnectTime[i] = Level.TimeSeconds;
			return false;
		}
	}
	
	return false; //just in case
}

event PreLogin(string Options, string Address, out string Error, out string FailCode)
{
	//local string ns;
	//
	//ns = ParseOption (Options, "natives");
	//
	//if(ns != "true")
	//{
	//	Error = "Please visit PlayDXMP.com to update your client!";
	//	return; //don't allow Engine.PreLogin to run, or it'll set Error to "" again
	//}
	
	// http://aluigi.altervista.org/adv/unrfs-adv.txt
    if(InStr(ParseOption(Options,"Class"),"%")!=-1 )
    {
        Error    = "CVE-2004-1805";
		log("attack detected ------------------------------------------------------------");
    }
	else if(bNoMonsters == true) //ANNAExt can't access us normally, so we're using an unused engine var :/
	{
		Error = "This server is not currently accepting new connections";
	}
	else
	{
		//check IP for reconnect spam (ban evasion)
		if(CheckIPSpam(Address) == true)
		{
			Error = "Too many reconnects. Please try again later!";
		}
		else
		{
			Super.PreLogin(Options, Address, Error, FailCode);
		}
	}
}

event PlayerPawn Login (string Portal, string URL, out string Error, Class<PlayerPawn> SpawnClass)
{
 	local MTLPlayer newPlayer;
	local string classStr, purgedClassStr;
	local int pkgLength;
	local Pawn PawnLink;
	local PlayerPawn TestPlayer;

	if((MaxPlayers > 0) && (NumPlayers >= MaxPlayers) )
	{
		Error=TooManyPlayers;
		return None;
	}

	for(PawnLink=Level.PawnList; PawnLink!=None; PawnLink=PawnLink.NextPawn)
	{
		TestPlayer = PlayerPawn(PawnLink);
		if(TestPlayer != None)
    {
      if(Len(TestPlayer.Password) == 0)
      {
        TestPlayer.Password = class'ANNAManager'.static.generateUID();
      }
		}
	}

	SpawnClass=DefaultPlayerClass;
	classStr=ParseOption(URL,"Class");
	pkgLength=InStr(classStr,".");
	if(pkgLength != -1 )
	{
		purgedClassStr = Mid(classStr,pkgLength + 1);
		classStr = Left(classStr,pkgLength);
	}
  else
  {
		purgedClassStr = classStr;
		classStr="";
	}
	if((purgedClassStr ~= "MPNSF") || (purgedClassStr ~= "MTLNSF") )
	  SpawnClass=class'ANNANSF';
    else if((purgedClassStr ~= "MPUNATCO") || (purgedClassStr ~= "MTLUNATCO") )
	  SpawnClass=Class'ANNAUNATCO';
    else if((purgedClassStr ~= "MPMJ12") || (purgedClassStr ~= "MTLMJ12") )
	  SpawnClass=class'ANNAMJ12';

	ChangeOption(URL,"Class",string(SpawnClass));
	newPlayer=MTLPlayer(super(DeathMatchGame).Login(Portal,URL,Error,SpawnClass));
	if(newPlayer != None )
	  newPlayer.V52(newPlayer.PlayerReplicationInfo.PlayerName);

	return newPlayer;
}

exec function ConsoleKick(int playerID)
{
  if(Manager == None)
    Manager = findManager();
  if(Manager == None)
    Manager = Spawn(class'ANNAManager');
    
  Manager.serverKick(playerID);
}

exec function ConsoleKickBan(int playerID)
{
  if(Manager == None)
    Manager = findManager();
  if(Manager == None)
    Manager = Spawn(class'ANNAManager');
    
  Manager.serverBan(playerID);
}

exec function ConsolePlayerList()
{
  if(Manager == None)
    Manager = findManager();
  if(Manager == None)
    Manager = Spawn(class'ANNAManager');
    
  Manager.serverPlayerList();
}

event PostLogin(playerpawn NewPlayer)
{
	local ANNAPlayer DXPlayer;

	Super.PostLogin(NewPlayer);

	DXPlayer = ANNAPlayer(NewPlayer);
	DXPlayer.AGM = AGM;
   
	if(DXPlayer != None)
	{
		DXPlayer.CheckInventory();
		AGM.PlayerJoin(DXPlayer);
	}
}

/**
  Disable haxy hax.
*/
function EditActor();

//called by Tick() *and* by Killed(). Checks if game should end
function bool CheckVictoryConditions( Pawn Killer, Pawn Killee, String Method )
{
	/*
	local Pawn winner;
	
	if(bHasGameFinished == false)
	{
		GetWinningPlayer( winner ); //maybe check even if finished? or call on gameMode instead? todo: 
		
		if ( VictoryCondition ~= "Frags" )
		{
			GetWinningPlayer( winner );

			if ( winner != None )
			{
				if (( winner.PlayerReplicationInfo.Score == ScoreToWin-(ScoreToWin/5)) && ( ScoreToWin >= 10 ))
					NotifyGameStatus( ScoreToWin/5, winner.PlayerReplicationInfo.PlayerName, False, False );
				else if (( winner.PlayerReplicationInfo.Score == (ScoreToWin - 1) ) && (ScoreTowin >= 2 ))
					NotifyGameStatus( 1, winner.PlayerReplicationInfo.PlayerName, False, True );

				if ( winner.PlayerReplicationInfo.Score >= ScoreToWin )
				{
					//PlayerHasWon( winner, Killer, Killee, Method );
					//return True;
					AGM.StopMatch(winner.PlayerReplicationInfo.PlayerName, 1, "");
					bHasGameFinished=true;
				}
			}
		}
		
		timeLimit = float(gameDuration)*60.0;

		if (( Level.Timeseconds >= timeLimit-NotifyMinutes*60.0 ) && ( timeLimit > NotifyMinutes*60.0*2.0 ))
			//NotifyGameStatus( int(NotifyMinutes), winner.PlayerReplicationInfo.PlayerName, True, True );

		if ( Level.Timeseconds >= timeLimit )
		{
			//PlayerHasWon( winner, Killer, Killee, Method );
			//return true;
			if(winner != none)
				AGM.StopMatch(winner.PlayerReplicationInfo.PlayerName, 1, "");
			else
				AGM.StopMatch("Game ended without a winner", 1, "");
				
			bHasGameFinished = true;
			//AGM.StopMatch(winner.PlayerReplicationInfo.PlayerName, 1, "Match ended after "$string(timeLimit * 60)$" minutes!");
		}
	}
	*/
	
	//we'll let GameMode do this stuff instead...
	
	return false;
}

/*
simulated function PlayerWinScreen( DeusExPlayer thisPlayer, GC gc, float screenWidth, float screenHeight, int winningTeam, String winnerName,
											  String killerStr, String killeeStr, String methodStr )
{
	local String str;
	local float x, y, w, h;

	// Show who won the match
	gc.SetTextColor( GoldColor );
	gc.SetFont(Font'FontMenuExtraLarge');

	if ( winningTeam == TEAM_DRAW )
		str = TeamDrawString;
	else
		//str = winnerName $ WonMatchString;
		AGM.GameWinnerStr("BOB", 1);

	gc.GetTextExtent( 0, w, h, str );
	x = (screenWidth * 0.5) - (w * 0.5);
	y = screenHeight * WinY;
	gc.DrawText( x, y, w, h, str );

	y += h;

	// Show who won it and who got killed
	if ( VictoryCondition ~= "Frags" )
	{
		gc.SetFont(Font'FontMenuTitle');
		if (( killerStr ~= "" ) || (killeeStr ~= "") || (methodStr ~=""))
			log( "Warning:Bad kill string in final death message." );
		else
		{
			str = MatchEnd1String $ killerStr $ MatchEnd2String $ killeeStr $ methodStr;
			gc.GetTextExtent( 0, w, h, str );
			if ( w >= screenWidth )
			{
				y -= (h * 0.5);
				str = MatchEnd1String $ killerStr $ MatchEnd2String $ killeeStr;
				gc.GetTextExtent( 0, w, h, str );
				x = (screenWidth * 0.5) - (w * 0.5);
				gc.DrawText( x, y, w, h, str );
				y += h;
				str = methodStr;
				gc.GetTextExtent( 0, w, h, str );
				x = (screenWidth * 0.5) - (w * 0.5);
				gc.DrawText( x, y, w, h, str );

			}
			else
			{
				x = (screenWidth * 0.5) - (w * 0.5);
				gc.DrawText( x, y, w, h, str );
			}
		}
	}

	// Show the scoreboard for all to see
	ShowDMScoreboard( thisPlayer, gc, screenWidth, screenHeight );

	// Press fire to continue message
	ContinueMsg( gc, screenWidth, screenHeight );
}*/

DefaultProperties
{
	GameTypeName="CDX Deathmatch")
	bHasGameFinished=false
    GameName="Deathmatch"
    DefaultPlayerClass=class'ANNAJCDenton'
    ScoreboardType=Class'ANNAScoreboard'
}
