// ============================================================
// PlayerId Mutator Version 2.05
//
// Author: Jon Steadman
// Email: AgentBionicman@aol.com
//
// Edited by Cozmo, 2018
// ============================================================
class MutPlayerId extends Mutator;// config(PlayerId);

//#exec TEXTURE IMPORT FILE=Classes\Symbol.bmp NAME=Beacon FLAGS=2

enum Align
{
	A_Left,				//align to the left
	A_Right,				//align to the right
	A_Center				//align to the center
};

struct PlayerNames
{
	var  bool bEnabled;		//display names or not
	var  bool bCheckForWalls;	//hide name if player is behind a wall
	var  bool bShowHealth;		//display health next to name in parenthesis
	var  Align Alignment;		//alignment of the name
	var  float Height;		//height above the player's head to draw the name
	var  Font Font;		//name will be displayed in this Font
	var  Color Color;		//color of name
	var  float CutOffDistance;	//distance at which to hide the name
};

struct PlayerSymbols
{
	var bool bEnabled;		//display the symbol or not
	var bool bCheckForWalls;	//hide symbol if player is behind a wall
	var float Height;		//height above the player's head to draw the symbol
	var Texture Symbol;		//the symbol
	var Align Alignment;		//alignment of the symbol
	var Color Color;		//color of symbol
	var float Scale;		//scale of symbol
	var float CutOffDistance;	//distance at which to hide the symbol
};

struct Graphics
{
	var PlayerNames NameOptions;
	var PlayerSymbols SymbolOptions;
};

var bool bEnabled;		//is this enabled on the client
var bool bCanClientDisable;	//can clients disable the mutator on their machine
var Graphics Team;		//Holds all information about drawing stuff for your team
var Graphics Enemy;		//Holds all information about drawing stuff for your enemy

var bool bCanDisable;
var Graphics TeamGraphics, EnemyGraphics;

var float refreshTimer;
var float refreshTime;

//
// Replication for Network
replication
{
	reliable if( bNetInitial && Role == ROLE_Authority )
		bCanDisable, TeamGraphics, EnemyGraphics;
}

//
// Actor has been created so set replicated variables to the default values.
function Spawned()
{
	bCanDisable = bCanClientDisable;
	TeamGraphics = Team;
	EnemyGraphics = Enemy;

	if( Level.NetMode == NM_ListenServer )
		SetTimer( 0.2, True );
}

simulated function PostNetBeginPlay()
{
	SetTimer( 0.2, True );
}

simulated function Timer()
{
	if( CreateWindow( class'winPlayerId' ) )
		SetTimer( 0.0, False );
}

//
// Create a window and set its clientObject.
simulated final function bool CreateWindow( class<Window> WinClass )
{
	local PlayerPawnExt P;
	local Window W;
	local TeamDMGame TDM;

	P = PlayerPawnExt( GetPlayerPawn() );

	if( P != None && P.rootWindow != None && WinClass != None )
	{
		W = P.rootWindow.NewChild( WinClass );

		if( W != None )
		{
			W.SetClientObject( Self );
			
			TDM = TeamDMGame(Level.Game);
			
			if(TDM != None)
				winPlayerId(W).bTeamGame = true;
			else
				winPlayerId(W).bTeamGame = false;
		}

		return( W != None );
	}

	return False;
}

function bool CheckReplacement( Actor Other, out byte bSuperRelevant )
{
	if( Other != None && Other != Self && Other.Class == Class )
		return False;
	else
		return true;
}

simulated final function bool IsDisabled()
{
	return bCanDisable && !bEnabled;
}

simulated final function Graphics GetTeamSettings()
{
	return TeamGraphics;
}

simulated final function Graphics GetEnemySettings()
{
	return EnemyGraphics;
}

//refreshing player net-relevancy to prevent weird "invisible" bug
/*
function Tick(float deltaTime)
{
	local Pawn curPawn;
	
	if(Level.TimeSeconds >= refreshTimer)
	{
		for(CurPawn = Level.PawnList; CurPawn != None; CurPawn = CurPawn.NextPawn)
        {
			if(curPawn.IsA('DeusExPlayer'))
			{
				if(curPawn.bAlwaysRelevant == False)
					curPawn.bAlwaysRelevant = True;
				else
					curPawn.bAlwaysRelevant = False;
			}
		}
		
		refreshTimer = Level.TimeSeconds + refreshTime;
	}
	
	Super.Tick(deltaTime);
}
*/

defaultproperties
{
    bEnabled=True
    Team=(NameOptions=(bEnabled=True,bCheckForWalls=True,bShowHealth=True,Alignment=A_Right,Height=20.000000,Font=Font'DeusExUI.FontMenuSmall',Color=(G=255),CutOffDistance=2000.000000),SymbolOptions=(bEnabled=True,Height=10.000000,Alignment=A_Center,Color=(G=255),Scale=0.080000,CutOffDistance=5000.000000))
    Enemy=(NameOptions=(bEnabled=True,bCheckForWalls=True,bShowHealth=True,Alignment=A_Center,Height=20.000000,Font=Font'DeusExUI.FontMenuSmall',Color=(R=255,G=128),CutOffDistance=2000.000000),SymbolOptions=(bEnabled=True,bCheckForWalls=True,Height=10.000000,Symbol=Texture'DeusExUI.Icons.DamageIconCenter',Alignment=A_Center,Color=(R=255,G=128,B=64),Scale=0.080000,CutOffDistance=5000.000000))
    bNetTemporary=True
    RemoteRole=ROLE_SimulatedProxy
    bAlwaysRelevant=True
	refreshTime=4.0;
}
