//======================================================================
// Draw info over players.
//
// Originally written by Jon Steadman, 2005. Modified by Cozmo, 2018
//======================================================================
class winPlayerId extends Window;

const DefaultTexture = Texture'DeusExUI.Icons.DamageIconDown';
const DefaultFont = Font'TechSmall';

struct Region
{
	var bool bEnabled;
	var int X;
	var int Y;
	var int W;
	var int H;
};

// ============================================================
// Properties
// ============================================================

var private float ResX, ResY;
var private PlayerPawnExt PC;
var private MutPlayerId M;
var bool bTeamGame;

// ============================================================
// InitWindow
// ============================================================

event InitWindow()
{
	SetWindowAlignments( HALIGN_Full, VALIGN_Full );
	ResX = width / 640.0;
	ResY = height / 480.0;
}

event ConfigurationChanged()
{
	SetWindowAlignments( HALIGN_Full, VALIGN_Full );
	ResX = width / 640.0;
	ResY = height / 480.0;
}


// ============================================================
// DrawWindow
// ============================================================

function DrawWindow(GC GC)
{
	local Pawn P;
	local GameInfo GI;

	if( GC == None )
		return;

	if( PC == None )
		PC = GetPlayerPawn();

	if( M == None )
		M = MutPlayerId(GetClientObject());

	if( M == None || M.IsDisabled() || PC == None || PC.bShowScores )
		return;
		
	//don't draw "teammates" in DM! But don't destroy this object (useful for cameras, etc)
	if(bTeamGame == true)
	{
		if(PC.PlayerReplicationInfo != None && PC.GameReplicationInfo != None )
		{
			foreach PC.AllActors(class'Pawn', P) //using level.pawnlist optimisation is difficult here
			{
				if((P != PC) && (PC.ViewTarget != P) && (P.Health > 0) && (!P.bHidden))
				{
					if((P.IsA('DeusExPlayer')) && (P.PlayerReplicationInfo != None))
					{
						if(P.PlayerReplicationInfo.Team == PC.PlayerReplicationInfo.Team)
						{
							DrawTeamGraphics(GC, P);
						}
					}
				}
			}
		}
	}
}


// ============================================================
// DrawTeamGraphics()
// ============================================================

function DrawTeamGraphics( GC C, Pawn Other )
{
	local bool bVisible;
	local float Px, Py, Scale, textwidth, textheight;
	local string S;
	local Region symbolRegion, nameRegion; //use a region so graphics wont overlap each other
	local font F;
	local texture T;

	bVisible = PC.LineOfSightTo(Other);
	symbolRegion.bEnabled = False;
	nameRegion.bEnabled = False;

	if( M.GetTeamSettings().SymbolOptions.bEnabled )
	{
		T = M.GetTeamSettings().SymbolOptions.Symbol;

		if( T == None )
			T = DefaultTexture;

		if( VSize(PC.Location - Other.Location) < M.GetTeamSettings().SymbolOptions.CutOffDistance )
		{
			if( ConvertVectorToCoordinates( Other.Location + vect( 0,0,1 ) * ( Other.CollisionHeight + M.GetTeamSettings().SymbolOptions.Height ), Px, Py) )
			{
				//if( !M.GetTeamSettings().SymbolOptions.bCheckForWalls || bVisible )
				//{
					Scale = FMax( 0.1, M.GetTeamSettings().SymbolOptions.Scale );
					symbolRegion.Y = Py - T.VSize * Scale * ResY;
					symbolRegion.W = T.USize * Scale * ResX;
					symbolRegion.H = T.VSize * Scale * ResY;

			//		switch( M.GetTeamSettings().SymbolOptions.Alignment )
			//		{
			//			case A_Left:
			//				symbolRegion.X = Px - T.USize * Scale * ResX;
			//				break;
			//			case A_Right:
			//				symbolRegion.X = Px;
			//				break;
			//			default://center
							symbolRegion.X = Px - T.USize * 0.5 * Scale * ResX;
			//				break;
			//		}

					C.SetTileColor(M.GetTeamSettings().SymbolOptions.Color);
					C.SetStyle(DSTY_Translucent);	
					C.DrawStretchedTexture( symbolRegion.X, symbolRegion.Y, symbolRegion.W, symbolRegion.H, 0, 0, T.USize, T.VSize, T );
					C.SetStyle(DSTY_Normal);
					symbolRegion.bEnabled = True;
				//}
			}
		}
	}
	
	Px = 0.0;
	Py = 0.0;

	if(M.GetTeamSettings().NameOptions.bEnabled )
	{
		if(Other.PlayerReplicationInfo != None)
			S = Other.PlayerReplicationInfo.PlayerName;
		else
			S = "Ally"; //temp. Todo: 

		if( M.GetTeamSettings().NameOptions.bShowHealth )
		{
		//	S = S$"|p1 ("$Other.Health$"%)";
			if(other.Health > 80)
				S = S$"|p1";
			else if(other.Health > 60)
				S = S$"|p1 - Injured";
			else
				S = S$"|p1 - Critical";
		}

		if( (S != "") && VSize(PC.Location - Other.Location) < M.GetTeamSettings().NameOptions.CutOffDistance )
		{
			if( ConvertVectorToCoordinates( Other.Location + vect(0,0,1) * ( Other.CollisionHeight + M.GetTeamSettings().NameOptions.Height ), Px, Py ) )
			{
				//if( !M.GetTeamSettings().NameOptions.bCheckForWalls || bVisible )
				//{
					F = M.GetTeamSettings().NameOptions.Font;

					if( F == None )
						F = DefaultFont;

					C.SetFont(F);
					C.SetTextColor(M.GetTeamSettings().NameOptions.Color);
					C.GetTextExtent(0.0, textwidth, textheight, S);
					nameRegion.Y = Py - textheight;
					nameRegion.W = textwidth;
					nameRegion.H = textHeight;

					//switch( M.GetTeamSettings().NameOptions.Alignment )
					//{
					//	case A_Left:
					//		nameRegion.X = Px - textwidth;
					//		break;
					//	case A_Right:
					//		nameRegion.X = Px;
					//		break;
					//	default:	//center
							nameRegion.X = Px - textwidth * 0.5;
					//		break;
					//}

					if( symbolRegion.bEnabled )
					{
						//adjust name so it wont overlap symbol
						if( M.GetTeamSettings().SymbolOptions.Height == M.GetTeamSettings().NameOptions.Height )
						{
							//name and symbol are side by side
							if( (M.GetTeamSettings().SymbolOptions.Alignment == A_Right && (M.GetTeamSettings().NameOptions.Alignment == A_Left || M.GetTeamSettings().NameOptions.Alignment == A_Center) ) || (M.GetTeamSettings().SymbolOptions.Alignment == A_Center && M.GetTeamSettings().NameOptions.Alignment == A_Left) )
								C.DrawText( symbolRegion.X - nameRegion.W - ResX, nameRegion.Y, nameRegion.W, nameRegion.H, S);//draw name on left side of symbol
							else
								C.DrawText( symbolRegion.X + symbolRegion.W + ResX, nameRegion.Y, nameRegion.W, nameRegion.H, S);//draw name on right side of symbol
						}
						else if( M.GetTeamSettings().SymbolOptions.Height > M.GetTeamSettings().NameOptions.Height )
						{
							//draw name below symbol
							C.DrawText( nameRegion.X, symbolRegion.Y + symbolRegion.H + ResY, nameRegion.W, nameRegion.H, S);
						}
						else
						{
							//draw name above symbol
							C.DrawText( nameRegion.X, symbolRegion.Y - nameRegion.H - ResY, nameRegion.W, nameRegion.H, S);
						}
					}
					else	//symbol is not drawn so just draw text as usual
						C.DrawText( nameRegion.X, nameRegion.Y, nameRegion.W, nameRegion.H, S);
				//}
			}
		}
	}
}


// ============================================================
// DestroyWindow()
// ============================================================

function DestroyWindow()
{
	SetClientObject(None);
	Super.DestroyWindow();
}

defaultproperties
{
}
